package pipe.gui;

import pipe.dataLayer.*;

import javax.swing.*;
import java.util.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import java.awt.event.MouseEvent;

import javax.swing.event.MouseInputAdapter;


/**
 * Class used to implement methods corresponding to mouse events on all PetriNetObjects.
 *
 */

public class PetriNetObjectHandler extends MouseInputAdapter implements Constants {


  protected Container contentPane;
  protected PetriNetObject tempPnobject;
  protected CreateGui theApp;
  protected boolean isDragging;
  protected boolean animationmode;

  // constructor passing in all required objects
  public PetriNetObjectHandler(CreateGui creategui, Container contentpane) {
    theApp = creategui;
    contentPane = contentpane;
    isDragging = false;

  }


  /** Creates the popup menu that the user will see when they right click on a component */
  public JPopupMenu getPopup(MouseEvent e) {

    JPopupMenu popup = new JPopupMenu();
    JMenuItem menuItem = new JMenuItem(new LabelAction(contentPane,e.getComponent()));
    menuItem.setText("Add label");
    popup.add(menuItem);

    menuItem = new JMenuItem(new DeleteAction(contentPane,e.getComponent(),theApp));
    menuItem.setText("Delete");
    popup.add(menuItem);

    return popup;
  }


  /** Displays the popup menu */
  private void maybeShowPopup(MouseEvent e) {
    if (e.isPopupTrigger()) {
      getPopup(e).show(e.getComponent(),
                       e.getX(), e.getY());
    }
  }


  /** Handler for mousePressed event - currently includes code for displaying a menu which the mouse is right clicked, or deleting/moving a component according to the buttons modeType (ie. which button has been selected at the top of the gui. */
  public void mousePressed(MouseEvent e) {
    if(ModeAdaptor.getMode() <=103){
      animationmode = true;
    }
    else {
      animationmode = false;
    }

    if(animationmode == true){
      return;
    }

    else {
      tempPnobject = (PetriNetObject)e.getComponent();
      Arc removeArc =null;
      boolean nameLabel = true;

      if (e.getButton() == MouseEvent.BUTTON3) {
        maybeShowPopup(e);
      }

      if (tempPnobject instanceof ArrowLabel) {
        removeArc = ((ArrowLabel)tempPnobject).getArc();
        nameLabel = false;
      }

      switch(ModeAdaptor.getMode()){
        case DELETE:
          theApp.getModel().removePetriNetObject(tempPnobject);
          if (removeArc != null) {
            theApp.getModel().removePetriNetObject(removeArc);
          }
          contentPane.repaint();
          tempPnobject = null;

          break;

        case MOVE:
          if (tempPnobject.getMovable()) {
            Insets insets = tempPnobject.getInsets();
            int eventX = e.getX();
            int eventY = e.getY();
          }
          break;

      }
    }

  }

  /** Event handler for when the user releases the mouse, used in conjunction with mouseDragged and mouseReleased to implement the moving action */

  public void mouseReleased(MouseEvent e) {

    if(animationmode==true){
      return;
    }

    else {
      if (e.getButton() == MouseEvent.BUTTON3) {
        maybeShowPopup(e);
      }

      tempPnobject = (PetriNetObject)e.getComponent();
      switch(ModeAdaptor.getMode()){
        case MOVE:
          if (tempPnobject.getMovable()) {
            if (isDragging)
              tempPnobject.updateSize(e);

            isDragging = false;
            break;
          }
      }
    }
  }


  /** Implements the moving of the objects */
  public void mouseDragged(MouseEvent e) {
    if(animationmode == true){
      return;
    }

    else {
      switch(ModeAdaptor.getMode()){
        case MOVE:
          if (tempPnobject.getMovable()) {
            //System.out.println("case move in dragging");
            tempPnobject = (PetriNetObject)e.getComponent();
            isDragging = true;
            tempPnobject.updateSize(e);
            break;
          }
      }
    }

  }
}














